function [A,B,C,f,fp,iter,tripcos,mintripcos,ttt]=cpfunc_fastzero(X,n,m,p,r,ort2,ort3,start,conv,B,C,maxit);
% [A,B,C,f,fp,iter,tripcos,mintripcos,ttt]=cpfunc_fastzero(X,n,m,p,r,ort2,ort3,start,conv,B,C,maxit);
% program candecomp/parafac, minimizes sum(k) || X(k) - A D(k) B' ||
% Fast computation for BIG matrices, for zero correlation constraint on A-mode
%
% Input: X: data
%        n,m,p: order
%        r: dimensionality of solution
%        ort2, ort3: constraints on A,B and C, resp., indicated by values
%                       1=No constraint
%						2=Orthogonality
%						3=Zero correlations
% 			start: 
%						0 = from SVD's
%						1 = random, orthonormalized
%						2 = user specified component matrices
%		 conv: convergence criterion, e.g., 1e-6
%        B,C: starting values for component matrices B and C (only if start=2)
%        maxit: maximal number of iterations
% Output:
%        A,B,C = component matrices
%        f = loss function value
%			fp = fit percentage
%        iter= number of iterations
%        tripcos = minimal triple cosine between two components, across three component matrices
%        ftiter= function values and minimal triple cosine at every 10 iterations 
%	      ttt=computation time 
% uses permnew.m ssq.m tr.m

mintripcos=0;

[nn,mp]=size(X);
if nn~=n | mp~=m*p, 
  beep
  fprintf(' Specified size of array (%g x %g x %g) does not correspond to actual size\n',n,m,p');
  keyboard
end;

t0=clock;

XX=X'*X;
ssX=tr(XX);
mX=mean(X);
XJX=XX-n*mX'*mX;

if start==0 
  % rational starts via eigendecompositions, or random if r is too big
%  if n>=r,[K,L]=ed(X*X');A=K(:,1:r);else A=orth(rand(r,r)-.5);A=A(1:n,:);end;
  Z=permnew(X,n,m,p);		% yields m x p x n array
  if m>=r,[K,L]=ed(Z*Z');B=K(:,1:r);else B=orth(rand(r,r)-.5);B=B(1:m,:);end;
  Z=permnew(Z,m,p,n);		% yields p x n x m
  if p>=r,[K,L]=ed(Z*Z');C=K(:,1:r);else C=orth(rand(r,r)-.5);C=C(1:p,:);end;
end;    

if start==1
%  if n>=r,A=orth(rand(n,r)-.5);else A=orth(rand(r,r)-.5);A=A(1:n,:);end;
  if m>=r,B=orth(rand(m,r)-.5);else B=orth(rand(r,r)-.5);B=B(1:m,:);end;
  if p>=r,C=orth(rand(p,r)-.5);else C=orth(rand(r,r)-.5);C=C(1:p,:);end;
end;

% S=X'*A; % mp x r matrix; COMPUTATIONAL INTENSIVE
% Z=A'*A; % r x r matrix; COMPUTATIONAL INTENSIVE
F=zeros(m*p,r);
for k=1:p 
    F((k-1)*m+1:k*m,:)=B*diag(C(k,:));
end;
% f=ssX-2*tr(F'*S)+tr(F'*F*Z);  
f=ssX;
fprintf(' SSQ(X) taken as function value at Start %12.8f \n',ssX);
fold=f+2*conv*f;
iter=0;
BB=B'*B;CC=C'*C;

while (fold-f>conv*f | iter<2) & f>conv^2 & iter<maxit

  fold=f;  
  
  % Update A, implicitly
  % first compute XF = X H C'xB' en FF' (implicitly)

  S=XJX*F*(F'*XJX*F)^-.5+n*mX'*mX/F';
  Z=eye(r)+n*(F\mX'*mX/F');

  % Evaluate
  %f=ssX-2*tr(F'*S)+tr(F'*F*Z);
  %if rem(iter,1)==0,fprintf('f= %12.8f after %g iters ; diff.=%12.8f \n',f,iter,(fold-f));end;
 
  % Update B
  
  G=zeros(m,r);
  H=zeros(r,r);
  for k=1:p 
    G=G+S((k-1)*m+1:k*m,:)*diag(C(k,:));
    if ort2~=2,H=H+diag(C(k,:))*Z*diag(C(k,:));end;
  end;
  if ort2==1, B=G/H;end;
  if ort2==2, [P,D,Q]=svd(G,0);B=P*Q';end;
  if ort2==3
    [P,D,Q]=svd(G-ones(m,1)*mean(G),0);
    B=P*Q'+ones(m,1)*mean(G)/H;
  end;
  BB=B'*B;

  % Evaluate
  %for k=1:p 
  %  F((k-1)*m+1:k*m,:)=B*diag(C(k,:));
  %end;
  %f=ssX-2*tr(F'*S)+tr(F'*F*Z);
  %if rem(iter,1)==0,fprintf('f= %12.8f after %g iters ; diff.=%12.8f \n',f,iter,(fold-f));end;

  % Update C
  
  if ort3~=2,K=Z.*BB;end;
  Y=zeros(p,r);
  for k=1:p
      d=diag(S((k-1)*m+1:k*m,:)'*B)';
      Y(k,:)=d;
  end;
  if ort3==1, C=Y/K;end;
  if ort3==2, [P,D,Q]=svd(Y,0);C=P*Q';end;
  if ort3==3
    [P,D,Q]=svd(Y-ones(p,1)*mean(Y),0);
    C=P*Q'+ones(p,1)*mean(Y)/K;
  end;
  CC=C'*C; 

  % Update F
  for k=1:p 
    F((k-1)*m+1:k*m,:)=B*diag(C(k,:));
  end;
  % Evaluate
  f=ssX-2*tr(F'*S)+tr(F'*F*Z);
  iter=iter+1;
  if rem(iter,50)==0,fprintf('f= %12.8f after %g iters ; diff.=%12.8f \n',f,iter,(fold-f));end;
end;

% Compute A by one additIonal update
  H=zeros(r,r^2);		% superidentity 3-way array
  for ii=1:r	
     H(ii,(ii-1)*r+ii)=1;
  end;
  Z1=permnew(X,n,m,p);
  Z1=permnew(B'*Z1,r,p,n);
  Z1=permnew(C'*Z1,r,n,r);   % gives n x r^2 array
  XF=Z1*H';
  FF=BB.*CC;
  [P,D,Q]=svd(XF-ones(n,1)*mean(XF),0);
  A=P*Q'+ones(n,1)*mean(XF)/FF;

ttt=etime(clock,t0);

fp=100-100*f/ssq(X);
tripcos=min(min(phi(A,A).*phi(B,B).*phi(C,C)));
if iter<10,mintripcos=tripcos;end;
fprintf(' Function value is %12.8f after %g iterations \n',f,iter);
fprintf(' Fit percentage is %6.2f %%\n',fp);
fprintf(' Procedure used %g seconds \n',ttt);
