function [A,B,C,f,fp,iter,tripcos,mintripcos,ttt]=cpfunc_fastzero2(X,n,m,p,r,ort2,ort3,start,conv,B,C,maxit);
% [A,B,C,f,fp,iter,tripcos,mintripcos,ttt]=cpfunc_fastzero2(X,n,m,p,r,ort2,ort3,start,conv,B,C,maxit);
% program candecomp/parafac, minimizes sum(k) || X(k) - A D(k) B' ||
% Fast computation for BIG matrices, for zero correlation constraint on A-mode
% Second procedure, based on projection
%
% Input: X: data
%        n,m,p: order
%        r: dimensionality of solution
%        ort2, ort3: constraints on A,B and C, resp., indicated by values
%                       1=No constraint
%						2=Orthogonality
%						3=Zero correlations
% 			start: 
%						0 = from SVD's
%						1 = random, orthonormalized
%						2 = user specified component matrices
%		 conv: convergence criterion, e.g., 1e-6
%        A,B,C: starting values for component matrices (only if start=2)
%        maxit: maximal number of iterations
% Output:
%        A,B,C = component matrices
%        f = loss function value
%			fp = fit percentage
%        iter= number of iterations
%        tripcos = minimal triple cosine between two components, across three component matrices
%        ftiter= function values and minimal triple cosine at every 10 iterations 
%	      ttt=computation time 
% uses permnew.m ssq.m tr.m

mintripcos=0;

[nn,mp]=size(X);
if nn~=n | mp~=m*p, 
  beep
  fprintf(' Specified size of array (%g x %g x %g) does not correspond to actual size\n',n,m,p');
  keyboard
end;

t0=clock;

% projection steps

mX=mean(X);
JX=X-ones(n,1)*mX;
XR=chol(JX'*JX);
XQ=JX/XR;
%[XQ,XR]=qr(JX,0);  % turns out to be relatively expensive, hence replaced
%by Cholesky
ssX=tr(XR'*XR)+n*mX*mX';
Rext=[XR;sqrt(n)*mX];   

if start==0 
  % rational starts via eigendecompositions, or random if r is too big
%  if n>=r,[K,L]=ed(X*X');A=K(:,1:r);else A=orth(rand(r,r)-.5);A=A(1:n,:);end;
  Z=permnew(X,n,m,p);		% yields m x p x n array
  if m>=r,[K,L]=ed(Z*Z');B=K(:,1:r);else B=orth(rand(r,r)-.5);B=B(1:m,:);end;
  Z=permnew(Z,m,p,n);		% yields p x n x m
  if p>=r,[K,L]=ed(Z*Z');C=K(:,1:r);else C=orth(rand(r,r)-.5);C=C(1:p,:);end;
end;    

if start==1
%  if n>=r,A=orth(rand(n,r)-.5);else A=orth(rand(r,r)-.5);A=A(1:n,:);end;
  if m>=r,B=orth(rand(m,r)-.5);else B=orth(rand(r,r)-.5);B=B(1:m,:);end;
  if p>=r,C=orth(rand(p,r)-.5);else C=orth(rand(r,r)-.5);C=C(1:p,:);end;
end;

% S=X'*A; % mp x r matrix; COMPUTATIONAL INTENSIVE
% Z=A'*A; % r x r matrix; COMPUTATIONAL INTENSIVE
F=zeros(m*p,r);
for k=1:p 
    F((k-1)*m+1:k*m,:)=B*diag(C(k,:));
end;
% f=ssX-2*tr(F'*S)+tr(F'*F*Z);  
f=ssX;
fprintf(' SSQ(X) taken as function value at Start %12.8f \n',ssX);
fold=f+2*conv*f;
iter=0;
BB=B'*B;CC=C'*C;
H=zeros(r,r^2);		% superidentity 3-way array
for ii=1:r	
  H(ii,(ii-1)*r+ii)=1;
end;

while (fold-f>conv*f | iter<2) & f>conv^2 & iter<maxit

  fold=f;  
  
  % Update A, implicitly by updating W and v
  % first compute XF = X H C'xB' en FF' (implicitly)

  % Update F
  for k=1:p 
    F((k-1)*m+1:k*m,:)=B*diag(C(k,:));
  end;

  [P,D,Q]=svd(XR*F,0);
  W=P*Q';
  v=F\mX';
  %A=XQ*W+ones(n,1)*v';  % real A

  % Evaluate

  %H1=permnew(H,r,r,r);
  %H1=permnew(B*H1,m,r,r);
  %H1=permnew(C*H1,p,r,m);  % gives r x m*p array  H*C'xB'
  %f=ssq(X-A*H1);
  %if rem(iter,1)==0,fprintf('f= %12.8f after %g iters ; diff.=%12.8f \n',f,iter,(fold-f));end;
 
  % Update B
  % first compute XF = X H A'xC' en FF' (implicitly)

  Wext=[W;sqrt(n)*v'];
  AA=Wext'*Wext;
  Zext=permnew(Rext,m*p+1,m,p);
  Z1=permnew(Zext,m,p,m*p+1);
  Z1=permnew(C'*Z1,r,m*p+1,m);
  Z1=permnew(Wext'*Z1,r,m,r);   % gives m x r^2 array
  XF=Z1*H'; % Computation of Rext*Fb
  if ort2==1
    FF=AA.*CC;  % Computation of Fb'Fb
    B=XF/FF;
  end;
  if ort2==2
    [P,D,Q]=svd(XF,0);
    B=P*Q';
  end;
  if ort2==3
    FF=AA.*CC;  % Computation of Fb'Fb
    [P,D,Q]=svd(XF-ones(m,1)*mean(XF),0);
    B=P*Q'+ones(m,1)*mean(XF)/FF;
  end;
  BB=B'*B;

  %H1=permnew(H,r,r,r);
  %H1=permnew(B*H1,m,r,r);
  %H1=permnew(C*H1,p,r,m);  % gives r x m*p array  H*C'xB'
  %f=ssq(X-A*H1);
  %fprintf('f= %12.8f after %g iters ; diff.=%12.8f \n',f,iter,(fold-f));

  % Update C
  % first compute XF = X H B'xA' en FF' (implicitly)

  Zext=permnew(Zext,m,p,m*p+1);
  Z1=permnew(Zext,p,m*p+1,m);
  Z1=permnew(Wext'*Z1,r,m,p);
  Z1=permnew(B'*Z1,r,p,r);   % gives p x r^2 array
  XF=Z1*H'; % Computation of Rext*Fc
  if ort3==1
    FF=AA.*BB;  % Computation of Fc'Fc
    C=XF/FF;
  end;
  if ort3==2
    [P,D,Q]=svd(XF,0);
    C=P*Q';
  end;
  if ort3==3
    FF=AA.*BB;  % Computation of Fc'Fc
    [P,D,Q]=svd(XF-ones(p,1)*mean(XF),0);
    C=P*Q'+ones(p,1)*mean(XF)/FF;
  end;
  CC=C'*C; 
  
  % Evaluate

  if ort3==1,f=ssX-tr(CC*FF);end;
  if ort3~=1,
    H1=permnew(H,r,r,r);
    H1=permnew(B*H1,m,r,r);
    H1=permnew(C*H1,p,r,m);  % gives r x m*p array  H*C'xB'
    f=ssq(Rext-Wext*H1);
  end;
  iter=iter+1;
  if rem(iter,50)==0,fprintf('f= %12.8f after %g iters ; diff.=%12.8f \n',f,iter,(fold-f));end;
end;
%compute A

A=XQ*W+ones(n,1)*v';
ttt=etime(clock,t0);

fp=100-100*f/ssq(X);
tripcos=min(min(phi(A,A).*phi(B,B).*phi(C,C)));
if iter<10,mintripcos=tripcos;end;
fprintf(' Function value is %12.8f after %g iterations \n',f,iter);
fprintf(' Fit percentage is %6.2f %%\n',fp);
fprintf(' Procedure used %g seconds \n',ttt);
